#ifndef SINUSSWEEPTORTSA_H
#define SINUSSWEEPTORTSA_H

#include <QObject>
#include <QNetworkAccessManager>
#include <QNetworkReply>
#include <QJsonDocument>
#include <QJsonObject>
#include <QTimer>

class SinusSweepToRTSA : public QObject
{
    Q_OBJECT
public:
	enum TxFormat {
		FLOAT32,
		INT16,
		ASCII
	};

	SinusSweepToRTSA(double centerFrequency = 2000000000, double sampleRate = 100000, int packetsPerSecond = 10, TxFormat txFormat = INT16);
    virtual ~SinusSweepToRTSA();

    void Start();

private slots:
	void onGeneratePacket();

    void onError(QNetworkReply::NetworkError code);
	void onFinished(QNetworkReply *);

private:
	void BuildFloat32Samples(double startTime, double stopTime, double sampleTime);
	void BuildInt16Samples(double startTime, double stopTime, double sampleTime);
	void BuildAsciiSamples(double startTime, double stopTime, double sampleTime);

	void postDataFloat32(QJsonDocument jdoc, float *samplesArray, int nSamples);
	void postDataInt16(QJsonDocument jdoc, int16_t *samplesArray, int nSamples);
	void postDataAscii(QJsonDocument jdoc);

    QNetworkAccessManager	*mNetworkAccessManager;
    double                  mCenterFrequency;
    double                  mSampleRate;
	int						mSamplesPerPacket;
	TxFormat				mTxFormat;

	qint64					mStreamStartTime;
	quint64					mSumSamples;
	long double				mLastPacketEnd;
    double                  mLastPhase;
    double                  mCurrentFrequency;
	double					mSampleResendTime;

    QTimer                  *mTimer;

    float                   *mSamplesArray;
    int16_t                 *mSamplesArrayInt16;
};

#endif // SINUSSWEEPTORTSA_H
